/*****************************************************************
Датчик APDS-9930 для определения приближения объектов и яркости света
http://iarduino.ru/shop/Sensory-Datchiki/datchik-osveschennosti-apds-9930.html
Данный скетч определяет значение регистров датчика
******************************************************************

ПОДКЛЮЧЕНИЕ ДАТЧИКА:

 Arduino | APDS-9930 | Назначение вывода
 --------+-----------+---------------------------------
   3.3V  |  VCC      | Напряжение питания
 --------+-----------+---------------------------------
   GND   |  GND      | Земля
 --------+-----------+---------------------------------
    A4   |  SDA      | Линия данных шины I2C
 --------+-----------+---------------------------------
    A5   |  SCL      | Линия тактирования шины I2C 
 --------+-----------+---------------------------------
         |  INT      | Выход прерывания (активный уровень «0»)
         |           | Вывод не используется в данном скетче

ВАЖНО: У датчика APDS-9930 логика и питание только 3,3 В
*****************************************************************/

// Подключаем библиотеки:                                       //
#include <Wire.h>                                               // Для работы с шиной I2C
#include <APDS9930.h>                                           // Для работы с датчиком APDS-9930
APDS9930 apds = APDS9930();                                     // Определяем объект apds, экземпляр класса APDS9930
                                                                //
// Объявляем переменные:                                        //
uint8_t reg;                                                    // Объявляем переменную для хранения номера   регистра
uint8_t val;                                                    // Объявляем переменную для хранения значения регистра
                                                                //
void setup(){                                                   //
    Serial.begin(9600);                                         // Инициируем передачу данных в монитор последовательного порта на скорости 9600 бит/сек
//  Инициируем работу датчика:                                  // 
    if(apds.init()){                                            // Если инициализация прошла успешно, то ...
          Serial.println("Initialization OK!");                 // Выводим сообщение об успешной инициализации датчика
    }else{Serial.println("Initialization ERROR!");}             // Иначе, выводим сообщение об ошибке инициализации датчика
                                                                //
//  Читаем и выводим значение регистров:                        // 
    for(reg = 0x00; reg <= 0x1E; reg++){                        // Проходим по регистрам с номерами от 0x00 до 0x1E включительно
        if(reg==0x10){reg=0x12;}                                // Пропускаем регистры с номерами 0x10, 0x11
        if(reg==0x1A){reg=0x1E;}                                // Пропускаем регистры с номерами 0x1A, 0x1B, 0x1C, 0x1D
        apds.wireReadDataByte(reg, val);                        // Читаем значение регистра с номером reg в переменную val
        Serial.print("REG 0x"); if(reg<0x10){Serial.print("0");} Serial.print  (reg, HEX); // Выводим номер    регистра в шестнадцатиричной форме
        Serial.print( " = 0x"); if(val<0x10){Serial.print("0");} Serial.println(val, HEX); // Выводим значение регистра в шестнадцатиричной форме
    }                                                           //
}                                                               //
                                                                //
void loop(){}                                                   // Код loop не задействован
