/*****************************************************************
Датчик APDS-9930 для определения приближения объектов и яркости света
http://iarduino.ru/shop/Sensory-Datchiki/datchik-osveschennosti-apds-9930.html
Данный скетч определяет яркость света
******************************************************************

ПОДКЛЮЧЕНИЕ ДАТЧИКА:

 Arduino | APDS-9930 | Назначение вывода
 --------+-----------+---------------------------------
   3.3V  |  VCC      | Напряжение питания
 --------+-----------+---------------------------------
   GND   |  GND      | Земля
 --------+-----------+---------------------------------
    A4   |  SDA      | Линия данных шины I2C
 --------+-----------+---------------------------------
    A5   |  SCL      | Линия тактирования шины I2C 
 --------+-----------+---------------------------------
         |  INT      | Выход прерывания (активный уровень «0»)
         |           | Вывод не используется в данном скетче

ВАЖНО: У датчика APDS-9930 логика и питание только 3,3 В
*****************************************************************/

// Подключаем библиотеки:                                       //
#include <Wire.h>                                               // Для работы с шиной I2C
#include <APDS9930.h>                                           // Для работы с датчиком APDS-9930
APDS9930 apds = APDS9930();                                     // Определяем объект apds, экземпляр класса APDS9930
                                                                //
// Объявляем переменные:                                        //
float    lightAmbient = 0;                                      // Определяем переменную для хранения освещённости общей                 в люксах
uint16_t lightCh0     = 0;                                      // Определяем переменную для хранения освещённости видимого и ИК спектра в люксах
uint16_t lightCh1     = 0;                                      // Определяем переменную для хранения освещённости инфракрасного спектра в люксах
                                                                //
void setup(){                                                   //
    Serial.begin(9600);                                         // Инициируем передачу данных в монитор последовательного порта на скорости 9600 бит/сек
//  Инициируем работу датчика:                                  // 
    if(apds.init()){                                            // Если инициализация прошла успешно, то ...
          Serial.println("Initialization OK!");                 // Выводим сообщение об успешной инициализации датчика
    }else{Serial.println("Initialization ERROR!");}             // Иначе, выводим сообщение об ошибке инициализации датчика
                                                                //
//  Разрешаем режим определения освещённости:                   //
    if(apds.enableLightSensor(false)){                          // Если режим определения освещённости запущен (false - без прерываний на выходе INT), то ...
          Serial.println("Start light sensor OK!");             // Выводим сообщение об успешном запуске режима определения освещённости
    }else{Serial.println("Start light sensor ERROR!");}         // Иначе, выводим сообщение об ошибке запуска режима определения освещённости
                                                                // Запретить режим определения освещённости можно так: bool j = apds.disableLightSensor(); // в переменную j сохранится результат выполнения функции (true/false)
//  Ждём завершение инициализации и калибровки:                 //
    delay(500);                                                 //
}                                                               //
                                                                //
void loop(){                                                    //
//  Читаем значения освещённости в переменные:                  //
    if(apds.readAmbientLightLux (lightAmbient)                  // Если прочитано значение общей освещённости в люксах
    && apds.readCh0Light        (lightCh0)                      // И    прочитано значение освещённости видимого и ИК спектра в люксах
    && apds.readCh1Light        (lightCh1)   ){                 // И    прочитано значение освещённости инфракрасного спектра в люксах, то ...
          Serial.println((String) "Ambient=" + lightAmbient + " lx,\t( IR/R/G/B=" + lightCh0 + ", IR=" + lightCh1 + " val )"); // Выводим все прочитанные значения
    }else{Serial.println("Read light ERROR!");}                 // Иначе, выводим сообщение об ошибке чтения освещённости
                                                                //
//  Приостанавливаем выполнение скетча на 1 секунду:            //
    delay(1000);                                                // Чтоб не перегружать шину I2C постоянными запросами
}                                                               //
